//:Main.java -- scenarios

/*
Code by Kirby Urner, 4D Solutions

Java 1.1 and above, last modified Dec 14, 1998

Educational freeware -- feel free to modify/recycle with all
due credit to your sources (i.e. follow the conventions of
scholarship).  If you modify any of the above package files
for public redistribution, please do so under a different
package name (com.pdx4d.polys should be uniquely the latest
4D Solutions distribution). This stuff is not guaranteed
to be bug free or purged of surprising behaviors.  Feel
free to send feedback to pdx4d@teleport.com re the code.

Read readme.txt if intending to unjar polys.jar in a
properly configured tree.

*/

import com.pdx4d.polys.*;

public class Main {
    static boolean help = false;
    static String outtype = "p";
    static int sw = 1;
    // to suppress printed messages, remove the V parameter
    // from the following 2 objects
    static POVwriter oPOVwriter   = new POVwriter("V");
    static VRMLwriter oVRMLwriter = new VRMLwriter("V");

    public static void main(String[] args){
        if (args.length==0) args = new String[]{"?"};
        if (args[0].equals("?")){
              help = true;
              System.out.println(" Help:");
              System.out.println("  | Enter: jre -cp .;polys.jar Main # c");
              System.out.println("  | # = valid scenario #; c = p|v|b (POV or VRML or both)");
              System.out.println("  | Defaults (if no args): # = 1, c = p");
              System.out.println("  | Default output: outfile.pov (Povray); outfile.wrl (VRML)");
              System.out.println("  | For this help enter with no # c parameters");
              System.out.println(" ");
              System.out.println(" Precoded scenarios:");
             }
         else {sw=Integer.valueOf(args[0]).intValue();}

         if(args.length == 2) outtype = args[1];

         // if (!help) System.out.println("Args: " + sw + " " + outtype);

         switch(sw){
            case(1): scenario1("  1.  "); if (!help) break;
            case(2): scenario2("  2.  "); if (!help) break;
            case(3): scenario3("  3.  "); if (!help) break;
            case(4): scenario4("  4.  "); if (!help) break;
            case(5): scenario5("  5.  "); if (!help) break;
            case(6): scenario6("  6.  "); if (!help) break;
            case(7): scenario7("  7.  "); if (!help) break;
            case(8): scenario8("  8.  "); if (!help) break;
            case(9): scenario9("  9.  "); if (!help) break;
            case(10): scenario10(" 10.  "); if (!help) break;
            case(11): scenario11(" 11.  "); if (!help) break;
            case(12): scenario12(" 12.  "); if (!help) break;
            case(13): scenario13(" 13.  "); if (!help) break;

            default: if (!help) scenario1("");}

    }

    private static void scenario1(String s){
        if (help)
           {System.out.println
            (s+"Testing translation/rotation");
            return;}
        // create objects
        Tetrahedron oTet = new Tetrahedron();
        Cube oCube = new Cube();
        VEaxes oRadials = new VEaxes();

        POVwriter.camfactor = 15;

        writeshape(oTet);
        writeshape(oRadials);
        oTet.translate(new double[]{2,1,1,0});
        oCube.translate(new double[]{2,1,1,0});

        writeshape(oTet);
        writeshape(oCube);

        oTet.rotate("X",90.0);
        oTet.setcolor("Black");
        writeshape(oTet);
        writedone();
    }

    private static void scenario2(String s){
        if (help)
           {System.out.println
           (s+"XYZ axes, Quadray axes");
            return;}
        // create objects
        Tetrahedron oTet = new Tetrahedron();
        XYZaxes xyz = new XYZaxes();
        xyz.setcolor("Green");
        Qaxes q = new Qaxes();
        q.setcolor("Red");


        // If POV is used
        POVwriter.camfactor = 8;

        writeshape(xyz);
        writeshape(q);
        writeshape(oTet);
        writedone();
     }

    private static void scenario3(String s){
        if (help)
           {System.out.println
           (s+"spheres in dodecas + cubocta");
            return;}
        // create objects
        RhDodecahedron oRD  = new RhDodecahedron();
        RhDodecahedron oRD2 = new RhDodecahedron();
        Cuboctahedron oCubocta = new Cuboctahedron();

        Sphere oSph  = new Sphere();
        Sphere oSph2 = new Sphere();
        System.out.println("Sphere volume: " + oSph2.shapevolume);

        oSph.setcolor("Green");
        oSph2.setcolor("Green");

        // If POV is used
        POVwriter.camfactor = 15;

        writeshape(oRD);
        writeshape(oSph);
        writeshape(oCubocta);

        // translate rh dodeca and sphere
        // and display
        oRD.translate(new double[]{2,1,1,0});
        oSph.translate(new double[]{2,1,1,0});
        writeshape(oRD);
        writeshape(oSph);

        // translate to other Rhdodeca, Sphere to other
        // corner of the cubocta, and display
        oRD2.translate(new double[]{2,1,0,1});
        oSph2.translate(new double[]{2,1,0,1});
        writeshape(oRD2);
        writeshape(oSph2);
        writedone();
    }

    private static void scenario4(String s){
        if (help)
           {System.out.println
            (s+"5-fold symmetry shapes + rhdodeca + sphere");
            return;}
        // create objects
        RhDodecahedron oRh = new RhDodecahedron();
        RhTriacontahedron oRT = new RhTriacontahedron("T");
        System.out.println("Emod factor: " + oRT.EMODFACTOR);
        System.out.println("Tmod factor: " + oRT.TMODFACTOR);
        Icosahedron oI = new Icosahedron();
        Dodecahedron oD = new Dodecahedron();

        oD.setcolor("Cyan","Orange");
        oD.vertradius = 0.10;
        oD.cylradius = 0.02;

        oI.setcolor("Cyan","Orange");
        oI.vertradius = 0.10;
        oI.cylradius = 0.02;

        Sphere oSph = new Sphere();

        // If POV is used
        POVwriter.camfactor = 8;

        writeshape(oRh);
        writeshape(oRT);
        writeshape(oI);
        writeshape(oD);
        writeshape(oSph);
        writedone();
    }

    private static void scenario5(String s){
        if (help)
           {System.out.println
            (s+"Simple Cubic = D + C (Kasman-Chu)");
            return;}
        // create objects
        RhDodecahedron oRd = new RhDodecahedron();
	Tetrahedron oTet = new Tetrahedron();
	Octahedron oOct = new Octahedron();
	Cube oCube = new Cube();
	Cube oCube2 = new Cube();
	Cube oCube3 = new Cube();
	Cuboctahedron oCuboct = new Cuboctahedron();
	VEaxes vx = new VEaxes();

        // Sphere sph1 = new Sphere();
        POVwriter.camfactor = 8;

	oRd.setcolor("Grey");
	vx.setcolor("Yellow","Blue");
	oCuboct.setcolor("Yellow","Blue");
        oOct.setcolor("Red","Blue");
        oTet.setcolor("Red");
	oCube.setcolor("Grey","Black");
	oCube2.setcolor("Black");
	oCube3.setcolor("Grey","Black");

        oOct.vertradius = 0.20;
	oCube.vertradius = 0.10;
        vx.vertradius = 0.20;
        oCuboct.vertradius = 0.20;
        // oTet.vertradius = 0.20;
 	oCube2.vertradius = 0.10;
 	oCube3.vertradius = 0.10;

 	oCube2.edgesvisible = false;

	// oTet.rotate("X",90.0);
	oCube2.scale(2);
	writeshape(oCube2);

	// writeshape(oCube);
        // writeshape(oRd);

        oCube.translate(new double[]{1,0,0,0});
	writeshape(oCube);
        oTet.translate(new double[]{1,0,0,0});
	//writeshape(oTet);

        oCube.translate(new double[]{0,1,1,2});
	writeshape(oCube);
        oCube.translate(new double[]{1,1,0,0});
	writeshape(oCube);
        oCube.translate(new double[]{0,1,1,0});
	writeshape(oCube);
        oCube.translate(new double[]{0,0,1,1});
	writeshape(oCube);

	oCube3.translate(new double[]{1,0,0,0});
        oCube3.translate(new double[]{0,1,1,0});
	writeshape(oCube3);
        oTet.translate(new double[]{0,1,1,0});
	oTet.rotate("X",90.0);
	//writeshape(oTet);

        oCube3.translate(new double[]{0,0,1,1});
	writeshape(oCube3);
        oTet.translate(new double[]{0,0,1,1});
        oTet.rotate("X",90.0);
	//writeshape(oTet);

        oCube3.translate(new double[]{1,0,0,1});
	writeshape(oCube3);
        oTet.translate(new double[]{1,0,0,1});
	oTet.rotate("X",90.0);
	//writeshape(oTet);

        oTet.translate(new double[]{0,1,0,1});
	oTet.rotate("X",90.0);
	//writeshape(oTet);

        oTet.translate(new double[]{1,1,0,0});
	oTet.rotate("X",90.0);
	//writeshape(oTet);

        oTet.translate(new double[]{0,1,1,0});
	oTet.rotate("X",90.0);
	//writeshape(oTet);

        oTet.translate(new double[]{0,0,1,1});
        oTet.rotate("X",90.0);
	//writeshape(oTet);

        oTet.translate(new double[]{1,0,0,1});
	oTet.rotate("X",90.0);
	//writeshape(oTet);


	//writeshape(oCuboct);
	//writeshape(vx);
	//writeshape(oOct);
        // writeshape(oTet);
        writedone();
    }

    private static void scenario6(String s){
        if (help)
           {System.out.println
            (s+"Face centered cubic (fcc)");
            return;}
        // create objects
        Octahedron o1 = new Octahedron();
        Cube c1 = new Cube();
        c1.setcolor("Grey","Black");
        o1.setcolor("Grey","Black");
        POVwriter.camfactor = 15;
        c1.vertradius=0.10;
        o1.vertradius=0.10;
        c1.scale(2);
        writeshape(c1);
        writeshape(o1);
        writedone();
    }

    private static void scenario7(String s){
        if (help)
           {System.out.println
            (s+"Curved mesh test");
            return;}
        // create objects
        Octahedron oct = new Octahedron();
        Icosahedron ico = new Icosahedron();

        POVwriter.camfactor = 18;
        oct.addmesh(3);
        writeshape(oct);

        oct.setcolor("Blue");
        oct.translate(new double[]{2,1,1,0});
        oct.curvemesh();
        oct.rotate("Z",30);
        oct.rotate("Y",45);
        oct.edgesvisible=false;
        writeshape(oct);

        ico.addmesh(6);
        ico.translate(new double[]{0,1,1,2});
        ico.curvemesh();
        ico.edgesvisible = false;
        writeshape(ico);

        writedone();
    }

    private static void scenario8(String s){
        if (help)
           {System.out.println
            (s+"Face centered cubic (fcc) and VE");
            return;}
        // create objects
        Octahedron o1 = new Octahedron();
        Octahedron o2 = new Octahedron();
        Octahedron o3 = new Octahedron();
        VEaxes vx = new VEaxes();
        Cube c1 = new Cube();
        Cuboctahedron ve1 = new Cuboctahedron();
        ve1.cylradius = 0.06;
        vx.cylradius = 0.06;
        vx.vertradius = 0.15;
        ve1.vertradius = 0.15;
        ve1.setcolor("Yellow","Orange");
        vx.setcolor("Yellow","Orange");
        c1.setcolor("Grey","Black");
        o1.setcolor("Grey","Black");
        o2.setcolor("Grey","Black");
        o3.setcolor("Grey","Black");
        POVwriter.camfactor = 15;
        c1.vertradius=0.10;
        o1.vertradius=0.10;
        o2.vertradius=0.10;
        o3.vertradius=0.10;
        c1.scale(2);
        writeshape(c1);
        writeshape(o1);
        o1.translate(new double[]{2,1,1,0});
        writeshape(o1);
        o2.translate(new double[]{1,0,2,1});
        o3.translate(new double[]{1,1,2,0});
        writeshape(o2);
        writeshape(o3);
        o2.translate(new double[]{2,1,0,1});
        writeshape(o2);
        ve1.translate(new double[]{1,0,1,0});
        vx.translate(new double[]{1,0,1,0});
        writeshape(ve1);
        writeshape(vx);
        writedone();
    }

    private static void scenario9(String s){
        if (help)
           {System.out.println
            (s+"A+B+C+D: the BCC ala Russ Kasman-Chu");
            return;}
        // create objects
	Tetrahedron oTet = new Tetrahedron();
	Tetrahedron oTet2 = new Tetrahedron();
	Cube oCube = new Cube();
	Cube oCube2 = new Cube();
	Cube oCube3 = new Cube();

        POVwriter.camfactor = 11;

        oTet.setcolor("Orange","Blue");
        oTet2.setcolor("Orange","Blue");
	oCube.setcolor("Grey","Black");
	oCube2.setcolor("Black");
	oCube3.setcolor("Grey","Black");

	oTet.vertradius = 0.10;
	oTet2.vertradius = 0.10;
	oCube.vertradius = 0.10;
 	oCube2.vertradius = 0.10;
 	oCube3.vertradius = 0.10;

 	oCube2.edgesvisible = false;

 	oTet2.edgesvisible = false;

	oTet2.rotate("X",90.0);
	oCube2.scale(2);
	writeshape(oCube2);

	// writeshape(oCube);

        oCube.translate(new double[]{1,0,0,0});
	writeshape(oCube);
        oCube.translate(new double[]{0,1,1,2});
	writeshape(oCube);
        oCube.translate(new double[]{1,1,0,0});
	writeshape(oCube);
        oCube.translate(new double[]{0,1,1,0});
	writeshape(oCube);
        oCube.translate(new double[]{0,0,1,1});
	writeshape(oCube);
	oCube3.translate(new double[]{1,0,0,0});
        oCube3.translate(new double[]{0,1,1,0});
	writeshape(oCube3);
        oCube3.translate(new double[]{0,0,1,1});
	writeshape(oCube3);
        oCube3.translate(new double[]{1,0,0,1});
	writeshape(oCube3);

        oTet.edgesvisible = false;
 	oTet2.edgesvisible = true;

	// oTet.setcolor("Black");
	oTet2.setcolor("Orange","Blue");

	writeshape(oTet);
	writeshape(oTet2);


        writedone();
    }

    private static void scenario10(String s){
        if (help)
           {System.out.println
            (s+"Testing trimeshes");
            return;}
        // create objects
        Sphere oSph = new Sphere();
        Icosahedron oIco = new Icosahedron();
        oIco.addmesh(6);
        oIco.setcolor("Magenta");
        oIco.edgesvisible=false;
        oIco.setdim(0.04,0.08);
        oIco.setcolor("Magenta","Blue");
        oIco.curvemesh();
        oSph.scale(1.8);
        POVwriter.camfactor = 9;
        writeshape(oIco);
        writeshape(oSph);
        writedone();
    }

    private static void scenario11(String s){
        if (help)
           {System.out.println
            (s+"Frequenced VE");
            return;}
        // create objects
	Icosahedron ico = new Icosahedron();
        XYZaxes xyz = new XYZaxes();

        POVwriter.camfactor = 10;

        //TXTwriter oTxtwriter = new TXTwriter("V");

        //tet.translate(new double[]{1,0,0,0});

        // writeshape(ico);

        //tet.rotate("Z",120);
        ico.setcolor("Green");

        ico.addmesh(10);
        ico.edgesvisible=false;
        //tet.setmeshcolor("Blue","Blue");

        ico.curvemesh();
        ico.oMesh.cymkmesh();
        ico.oMesh.meshvisible(false);
        ico.oMesh.setmeshdim(0.1,0.2);

        writeshape(ico);
        writeshape(xyz);

        //oTxtwriter.writeshape(tet);
        //oTxtwriter.writedone();

        //writeshape(ico);

        // writeshape(oCub);

        writedone();
    }

    private static void scenario12(String s){
        if (help)
           {System.out.println
            (s+"14 rays through VE face centers");
            return;}
        // create objects
        Cuboctahedron oCub = new Cuboctahedron();
        Tetrahedron t1 = new Tetrahedron();
        Tetrahedron t2 = new Tetrahedron();
        t2.rotate("X",90);
        Qaxes q1 = new Qaxes();
        Qaxes q2 = new Qaxes();
        q2.rotate("X",90);
        XYZaxes xyz = new XYZaxes();
        xyz.scale(1.5);
        q1.scale(2.0);
        q2.scale(2.0);
        xyz.setcolor("Blue");
        q1.setcolor("Blue");
        q2.setcolor("Blue");

        oPOVwriter.camfactor = 10;
        writeshape(t1);
        writeshape(t2);
        writeshape(q1);
        writeshape(q2);
        writeshape(xyz);
        writeshape(oCub);
        writedone();
    }

    private static void scenario13(String s){
        if (help)
           {System.out.println
            (s+"Testing synchwith method.");
            return;}
        // create objects
        Cube c1 = new Cube();
        c1.rotate("X",45);
        c1.rotate("Y",400);
        c1.rotate("Z",25);
        c1.translate(new double[]{1,2,1});
        Tetrahedron t1 = new Tetrahedron();
        t1.synchwith(c1);
        Octahedron o1 = new Octahedron();
        o1.synchwith(t1);
        oPOVwriter.camfactor=14;
        writeshape(t1);
        writeshape(o1);
        writeshape(c1);
        writedone();
    }

    private static void writeshape(Shape s){
        if (outtype.equals("p") | outtype.equals("P")) oPOVwriter.writeshape(s);
        if (outtype.equals("v") | outtype.equals("V")) oVRMLwriter.writeshape(s);
        if (outtype.equals("b") | outtype.equals("B")) {
             oPOVwriter.writeshape(s);
             oVRMLwriter.writeshape(s);
        }
    }

    private static void writedone(){
        if (outtype.equals("p") | outtype.equals("P")) oPOVwriter.writedone();
        if (outtype.equals("v") | outtype.equals("V")) oVRMLwriter.writedone();
        if (outtype.equals("b") | outtype.equals("B")){
             oPOVwriter.writedone();
             oVRMLwriter.writedone();
        }
    }
}

