//:Formats.java -- defines static coordinate conversion methods
package com.pdx4d.polys;

abstract class Formats {
    static final double ROOT2 = Math.sqrt(2);

    static public double[][] xyz2quad(double[][] points){
       double[][] quad = new double[points.length][4];
       for (int i=0;i<points.length;i++)
           quad[i]=xyz2quad(points[i]);
       return quad;
    }

    static public double[] xyz2quad(double[] a){
	double[] quadout = new double[4];
        double x = a[0];
        double y = a[1];
        double z = a[2];
        quadout[0] = 1/ROOT2 * ((x>=0? x: 0)+(y>=0? y: 0)+(z>=0? z: 0));
        quadout[1] = 1/ROOT2 * ((x>=0? 0:-x)+(y>=0? 0:-y)+(z>=0? z: 0));
        quadout[2] = 1/ROOT2 * ((x>=0? 0:-x)+(y>=0? y: 0)+(z>=0? 0:-z));
        quadout[3] = 1/ROOT2 * ((x>=0? x: 0)+(y>=0? 0:-y)+(z>=0? 0:-z));
        quadout = simplify(quadout);
	return quadout;
    }

    static private double[] simplify(double[] q){
       double minval= q[0];
       // get the least quadray value
       for (int i=1;i<4;i++) minval = minval<q[i]?minval:q[i];
       // subtract it from itself (=0) and all others
       for (int i=0;i<4;i++) q[i]=q[i]-minval;
       return q;
    }

    static public double[][] quad2xyz(double[][] points){
       double[][] xyz = new double[points.length][3];
       for (int i=0;i<points.length;i++)
           xyz[i]=quad2xyz(points[i]);
       return xyz;
    }

    static public double[] quad2xyz(double[] a){
        double[] xyzout = new double[3];	
        xyzout[0] = 1/ROOT2 * (a[0] - a[1] - a[2] + a[3]);
        xyzout[1] = 1/ROOT2 * (a[0] - a[1] + a[2] - a[3]);
        xyzout[2] = 1/ROOT2 * (a[0] + a[1] - a[2] - a[3]);
	return xyzout;
    }


    static public double[][] xyz2polar(double[][] points){
       double[][] polar = new double[points.length][3];
       for (int i=0;i<points.length;i++)
          polar[i]=xyz2polar(points[i]);
       return polar;
    }

    static public double[] xyz2polar(double[] a){
      double radius = 0;
      double theta = 0;
      double phi = 0;
      double xydist = 0;
      
      double x = a[0];
      double y = a[1];
      double z = a[2];

      radius = Math.sqrt(x*x + y*y + z*z);

      if (x!=0){
        if (x>0){ theta = Math.atan(y/x);}
        else    { theta = Math.atan(y/x) + Math.PI;}
      } else {
        if (y!=0) {
            if (y>0) {theta = Math.PI/2;}
            else     {theta = -Math.PI/2;}
        } else {
          theta = 0;
        }
     }

     if (z!=0){
        xydist = Math.sqrt(x*x + y*y);
        phi = Math.atan(z/xydist);
     } else {
       phi = 0;
     }
     
     double[] polarout = new double[3];
     polarout[0] = radius;
     polarout[1] = theta;
     polarout[2] = phi;

     return polarout;
   }

   static public double[][] polar2xyz(double[][] points){
       double[][] xyz = new double[points.length][3];
       for (int i=0;i<points.length;i++)
            xyz[i]=polar2xyz(points[i]);
       return xyz;
   }

   static public double[] polar2xyz(double[] a){
     double radius = a[0];
     double theta = a[1];
     double phi = a[2];
 
     double xydist = radius * Math.cos(phi);

     double[] xyzout = new double[3];
     xyzout[0] = xydist * Math.cos(theta);
     xyzout[1] = xydist * Math.sin(theta);
     xyzout[2] = radius * Math.sin(phi);

     return xyzout;
   }

   static public double[][] quad2polar(double[][] points){
       double[][] polar = new double[points.length][3];
       for (int i=0;i<points.length;i++)
            polar[i]=quad2polar(points[i]);
       return polar;
   }

   static public double[] quad2polar(double[] a){
	return xyz2polar(quad2xyz(a));
   }

   static public double[][] polar2quad(double[][] points){
       double[][] quad = new double[points.length][4];
       for (int i=0;i<points.length;i++)
              quad[i]=polar2quad(points[i]);
       return quad;
   }

   static public double[] polar2quad(double[] a){
	return xyz2quad(polar2xyz(a));
   }

   public static void main(String[] args){}
}
