//:Matrix.java -- defines Matrix methods
package com.pdx4d.polys;

public class Matrix{
    private double theta=0;
    private double cos_theta=0;
    private double sin_theta=0;
    private String axis="X";
    private Shape s;
    private double xrotate=0.0, yrotate=0.0, zrotate=0.0;
    private static final double DEG2RAD = Math.PI/180;
    private static final double RAD2DEG = 180/Math.PI;

    Matrix(Shape shape){
          s = shape;
          }

    public void scale(double scalefactor){
          for (int i=0;i<s.points.length;i++){
            s.points[i][0] = s.points[i][0] * scalefactor;
            s.points[i][1] = s.points[i][1] * scalefactor;
            s.points[i][2] = s.points[i][2] * scalefactor;}
    }

    public void translate(double[] t){
        for (int i=0;i<s.points.length;i++){  
            s.points[i][0] = s.points[i][0] + t[0];
            s.points[i][1] = s.points[i][1] + t[1];
            s.points[i][2] = s.points[i][2] + t[2];}
    }

    public void rotate(String ax, double degrees){
       setdegrees(ax, degrees);
       if (axis=="X") xrotate();
       if (axis=="Y") yrotate();
       if (axis=="Z") zrotate();
    }   

    public void rotate(){
       if (axis=="X") xrotate();
       if (axis=="Y") yrotate();
       if (axis=="Z") zrotate();
    }   

    public void setdegrees(String a, double d){
       axis = a;
       theta = DEG2RAD * d;
       cos_theta  = Math.cos(theta);
       sin_theta  = Math.sin(theta);
    }

    private void xrotate(){
          double newx;
          double newy;
          double newz;
          //         / 1   0        0       \
          // X AXIS  | 0   cos(a)  -sin(a)  |
          //         \ 0   sin(a)   cos(a)  /
          for (int i=0;i<s.points.length;i++){
             newy = cos_theta*s.points[i][1] - sin_theta*s.points[i][2];
             newz = sin_theta*s.points[i][1] + cos_theta*s.points[i][2];
             s.points[i][1] = newy;
             s.points[i][2] = newz;
          }
	  s.orient[0] = s.orient[0]+theta*RAD2DEG;
	  s.orient[0] = s.orient[0]%360.0;
    }

    private void yrotate(){
          double newx;
          double newy;
          double newz;
          //         / cos(a)   0  -sin(a) \
          // X AXIS  | 0        1  -sin(a) |
          //         \ sin(a)   0   cos(a) /
          for (int i=0;i<s.points.length;i++){
             newx = cos_theta*s.points[i][0] - sin_theta*s.points[i][2];
             newz = sin_theta*s.points[i][0] + cos_theta*s.points[i][2];
             s.points[i][0] = newx;
             s.points[i][2] = newz;
          }
	  s.orient[1] = s.orient[1]+theta*RAD2DEG;
	  s.orient[1] = s.orient[1]%360.0;
    }

    private void zrotate(){
          double newx;
          double newy;
          //         / cos(a)  -sin(a)  0 \
          // Z AXIS  | sin(a)   cos(a)  0 |
          //         \ 0             0  1 /
          for (int i=0;i<s.points.length;i++){
             newx = cos_theta*s.points[i][0] - sin_theta*s.points[i][1];
             newy = sin_theta*s.points[i][0] + cos_theta*s.points[i][1];
             s.points[i][0] = newx;
             s.points[i][1] = newy;
             }
	  s.orient[2] = s.orient[2]+theta*RAD2DEG;
	  s.orient[2] = s.orient[2]%360.0;
    }

    public double dist(double[] vector){
          double x = vector[0];
          double y = vector[1];
          double z = vector[2];
          return Math.sqrt(x*x + y*y + z*z);
    }


    public static void main(String[] args){}

}
