//:POVwriter.java -- defines POV subclass of DataWriter
package com.pdx4d.polys;

import java.io.*;

public class POVwriter extends DataWriter {
    public static double camfactor = 8;

    public POVwriter(){ 
         thisWriter = this;
         outfile = new String("outfile.pov");
    }

    public POVwriter(String s){
        if (s.equals("V")) mute = false;  // verbose operations
        thisWriter = this;
        outfile = new String("outfile.pov");
    }

    public void writeshape(Shape s){
         writeshape(s, thisWriter); // pass up to superclass w/ this
    }

    protected void writeheader(){
      if (!mute) System.out.println("Write POV header");
      output.println("//POV-Ray script");
      output.println("#version 3.1;");
      output.println("global_settings { assumed_gamma 2.2 }");
      output.println("#include \"colors.inc\"");
      output.println("");
      output.println("#declare Cam_factor = " + camfactor + ";");
      output.println("#declare Camera_X = 1 * Cam_factor;");
      output.println("#declare Camera_Y = 0.5 * Cam_factor;");
      output.println("#declare Camera_Z = -0.9 * Cam_factor;");
      output.println("camera { location  <Camera_X, Camera_Y, Camera_Z>");
      output.println("            up        <0, 1.0,  0>    right     <-4/3, 0,  0>");
      output.println("            direction <0, 0,  3>      look_at   <0, 0, 0> ");
      output.println("            rotate <0,0,0>}");
      output.println("");
      output.println("light_source { <Camera_X - 2, Camera_Y + 5 , Camera_Z + 5> color White }");
      output.println("light_source { <Camera_X - 2, Camera_Y + 5 , Camera_Z - 3> color White }");
      //output.println("light_source { <Camera_X + 2, Camera_Y - 5 , Camera_Z - 3> color White }");  
      //output.println("light_source { <Camera_X + 4, Camera_Y - 6 , Camera_Z - 3> color White }");
      output.println("");
      output.println("// Background:");
      output.println("background {color White}");    
    }

    protected void writeEdges(Shape s){
       nubtickoff = new int[s.points.length];

       for (int i=0;i<s.edges.length;i++){
          int vert1 = s.edges[i][0];
          int vert2 = s.edges[i][1];
 
          if (nubtickoff[vert1]==0){
               writeNub(s,vert1);
               nubtickoff[vert1]=1;
          }

          if (s.edgesvisible) writeCylinder(s,vert1,vert2);

          if (nubtickoff[vert2]==0){
               writeNub(s,vert2);
               nubtickoff[vert2]=1;
          }
       }
    }

    protected void writeNub(Shape s,int pointindex){
        double a = s.points[pointindex][0];
        double b = s.points[pointindex][1];
        double c = s.points[pointindex][2];
	String vcolor;
	if (s.colorcoded){
            vcolor = getcolor(s,pointindex);
        } else {
            vcolor = s.vertcolor;
        }
        writesphere(a,b,c,s.vertradius, vcolor);
    }

    protected String getcolor(Shape s,int pointindex){
        double r = s.rgbcodes[pointindex][0];
        double g = s.rgbcodes[pointindex][1];
        double b = s.rgbcodes[pointindex][2];
	String rgb = "rgb<"+r+","+g+","+b+">";
	return rgb;
    }

    protected void writeCylinder
        (Shape s,int startpoint, int endpoint){

        double a = s.points[startpoint][0];
        double b = s.points[startpoint][1];
        double c = s.points[startpoint][2];
        double d = s.points[endpoint][0];
        double e = s.points[endpoint][1];
        double f = s.points[endpoint][2];

        output.println(
            "cylinder{<"+ a +"," + b + "," + c +">,<"
                        + d +"," + e + "," + f +">," + s.cylradius
                        + " pigment {color " + s.edgecolor + "} no_shadow}");        
    }

    protected void writesphere(Shape s){
       double x = s.pivot[0];
       double y = s.pivot[1];
       double z = s.pivot[2];
       double rad = s.radius;
       String color = s.edgecolor;
       writesphere(x,y,z,rad,color);
    }

    protected void writesphere
    (double x, double y, double z, double rad, String color){

       if (color.equals("Brown")){
          output.println("sphere { <" + x+", "+ y+", "+ z+">, " + rad);
          output.println("      texture {");
          output.println("        pigment {");
          output.println("           wood");
          output.println("           color_map {");
          output.println("            [0.0 color DarkTan]");
          output.println("            [0.9 color DarkBrown]");
          output.println("            [1.0 color VeryDarkBrown]");
          output.println("                     }");
          output.println("        turbulence 0.08");
          output.println("                 }" );
          output.println("      finish { phong .5 }");
          output.println("             }");
          output.println("no_shadow }");
       } else {
          output.println("sphere { <" + x+", "+ y+", "+ z+">, " + rad);
          output.println("      texture {");
          output.println("       pigment {color " + color +"}");
          output.println("       finish { phong .5 }");
          output.println("              }");
          output.println("no_shadow }");
       } 
    }

    public static void main(String[] args){}
}
