//:Sqrmesh.java -- extends Mesh class, creates square
// mesh of specified frequency (default = 1)

package com.pdx4d.polys;

public class Sqrmesh extends Mesh {  

    public Sqrmesh(Shape s,int[] face,int freq){
       shapefiletype="X";
       setcolor("Grey");
       shapevolume = 0;  
       setcorners(s,face);
       setfreq(freq);
    }

    public void setfreq(int f) {
       frequency = f;
       // square # = (frequency+1)^2
       noverts = (frequency+1) * (frequency+1);
       // V+F = E+1 so E=V+F-1 where F = frequency^2
       noedges = noverts + (frequency*frequency) - 1;
       calcpoints();
       calcedges();
    }
    
    private void calcedges() {

       edges  = new int[noedges][2];

       int lastinrow   = -1;
       int rownum      = 0;
       int term        = 0;
       int termsinrow  = 1;
       int firstterm   = 0;
       int edgeno      = 0;

       while (rownum < frequency + 1){

             firstterm   = lastinrow + 1;
             lastinrow   = firstterm + frequency;
             termsinrow  = frequency + 1;
 
             term = firstterm;
             while (term < lastinrow){
                edges[edgeno][0]=term;
                edges[edgeno][1]=term+1;
                edgeno++;
                term++;
              }

              term = firstterm;
              if (rownum < frequency){
                 while (term < lastinrow + 1){
                   edges[edgeno][0]=term;
                   edges[edgeno][1]=term+frequency+1;
                   edgeno++;
                   term++;
                 }
              }  
              rownum++;              
       }
    }

    protected void calcpoints() {

       points = new double[noverts][3];

       for (int i=0;i<4;i++){
          for (int j=0;j<3;j++){
             points[i][j] = xyzcorners[i][j];
          }
       }

       double[] abstep = new double[3];
       double[] bcstep = new double[3];
      
       for (int i=0;i<3;i++){
          abstep[i] = (xyzcorners[1][i] - xyzcorners[0][i])/frequency;
          bcstep[i] = (xyzcorners[2][i] - xyzcorners[1][i])/frequency;
       }

       int pointno     = 0;
       int lastinrow   = -1;
       int rownum      = 0;
       int termsinrow  = frequency + 1;
       int firstterm   = 0;
       int term        = 0;
       int edgeno      = 0;

       while (rownum < frequency + 1){

             firstterm   = lastinrow + 1;
             lastinrow   = firstterm + frequency;
 
             term = firstterm;
             while (term < lastinrow){
                for (int i = 0; i<3; i++){
                   points[pointno+1][i] = points[pointno][i] + abstep[i];
                }
                pointno++;
                term++;
              }
  
              rownum++;
              pointno++;
              
              if (pointno < noverts){
                 for (int i = 0;i<3;i++){
                     points[pointno][i] = points[pointno - termsinrow][i] + bcstep[i];
                 }
              }    
       }
    }

    public static void main(String[] args){
    }
}
