//:TXTwriter.java -- defines TXT subclass of DataWriter
package com.pdx4d.polys;

import java.io.*;

public class TXTwriter extends DataWriter {
    private int tuples = 3;
    private String filetype = "X";

    public TXTwriter(){ 
         thisWriter = this;
         outfile = new String("outfile.txt");
    }

    public TXTwriter(String s){
        if (s.equals("V")) mute = false;  // verbose operations
        thisWriter = this;
        outfile = new String("outfile.txt");
    }

    public void writeshape(Shape s){
	filetype = s.shapefiletype;
        if (filetype.equals("Q")) tuples = 4;
        writeshape(s, thisWriter); // pass up to superclass w/ this
    }

    protected void writeheader(){
      if (filetype.equals("Q")) {
          if (!mute) System.out.println("Write Quadrays header");
          output.println("Quadray Coordinates");
      }
      if (filetype.equals("P")) {
          if (!mute) System.out.println("Write Polar header");
          output.println("Polar Coordinates");
      }
      if (filetype.equals("X")) {
          if (!mute) System.out.println("Write XYZ header");
          output.println("XYZ Coordinates");
      }
    }

    protected void writeEdges(Shape s){
       nubtickoff = new int[s.points.length];

       for (int i=0;i<s.edges.length;i++){
          int vert1 = s.edges[i][0];
          int vert2 = s.edges[i][1];
 
          if (nubtickoff[vert1]==0){
               writeNub(s,vert1);
               nubtickoff[vert1]=1;
          }

          if (s.edgesvisible) writeCylinder(s,vert1,vert2);

          if (nubtickoff[vert2]==0){
               writeNub(s,vert2);
               nubtickoff[vert2]=1;
          }
       }
    }

    protected void writeNub(Shape s,int pointindex){
	if (tuples==3){

        double a = s.points[pointindex][0];
        double b = s.points[pointindex][1];
        double c = s.points[pointindex][2];
        writesphere(a,b,c,s.vertradius,s.vertcolor);

	} else {

        double a = s.points[pointindex][0];
        double b = s.points[pointindex][1];
        double c = s.points[pointindex][2];
        double d = s.points[pointindex][3];
        writesphere(a,b,c,d,s.vertradius,s.vertcolor);

	}

    }

    protected void writeCylinder
        (Shape s,int startpoint, int endpoint){

	if (tuples==3){

        double a = s.points[startpoint][0];
        double b = s.points[startpoint][1];
        double c = s.points[startpoint][2];

        double d = s.points[endpoint][0];
        double e = s.points[endpoint][1];
        double f = s.points[endpoint][2];

        output.println(
             "Edge from ("+ a +"," + b + "," + c + ") to ("
                          + d +"," + e + "," + f +")");

	} else {

        double a = s.points[startpoint][0];
        double b = s.points[startpoint][1];
        double c = s.points[startpoint][2];
        double d = s.points[startpoint][3];

        double e = s.points[endpoint][0];
        double f = s.points[endpoint][1];
        double g = s.points[endpoint][2];
        double h = s.points[endpoint][3];

        output.println(
             "Edge from ("+ a +"," + b + "," + c + "," + d + ") to ("
                          + e +"," + f + "," + g + "," + h +")");
        }
    }

    protected void writesphere(Shape s){
       double x = s.pivot[0];
       double y = s.pivot[1];
       double z = s.pivot[2];
       double rad = s.radius;
       String color = s.edgecolor;
       writesphere(x,y,z,rad,color);
    }

    protected void writesphere
    (double x, double y, double z, double rad, String color){
        output.println(
             "Sphere at ("+ x +"," + y + "," + z + ") Radius =" + rad);
    }

    protected void writesphere
    (double a, double b, double c, double d, double rad, String color){
        output.println(
             "Sphere at ("+ a +"," + b + "," + c + "," + d + ") Radius =" + rad);
    }

    public static void main(String[] args){}
}
