//:VRMLwriter.java -- defines VRMLWriter
package com.pdx4d.polys;

import java.io.*;
import java.util.*;

public class VRMLwriter extends DataWriter {
    protected static double[][] vrmlcyls;
    private static boolean mute = true;
    public static ColorDict colorstrings;

    public VRMLwriter() {
        colorstrings = new ColorDict();
        thisWriter = this;
        outfile = new String("outfile.wrl");
    }

    public VRMLwriter(String s) {
        if (s.equals("V")) mute = false; // verbose setting (print messages)
        colorstrings = new ColorDict();
        thisWriter = this;
        outfile = new String("outfile.wrl");
    }

    public void writeshape(Shape s){
         writeshape(s, thisWriter); // pass up to superclass w/ this
    }

    protected void writeheader(){
      if (!mute) System.out.println("Write VRML header");
      output.println("#VRML V1.0 ascii");
      output.println("DEF BackgroundColor Info { string \"1.0 1.0 1.0\" }");
      output.println("Material { diffuseColor 0.0 1.0 0.0}  #end Material set to white");
      output.println("  ");
    }

    protected void writeEdges(Shape s){
       vrmlcyls = Vrmlops.vrmlcalcs(s);
       nubtickoff = new int[s.points.length];

       for (int i=0;i<s.edges.length;i++){
          int vert1 = s.edges[i][0];
          int vert2 = s.edges[i][1];
 
          if (nubtickoff[vert1]==0){
               writeNub(s,vert1);
               nubtickoff[vert1]=1;
          }

          if (s.edgesvisible) writeCylinder(s,i);

          if (nubtickoff[vert2]==0){
               writeNub(s,vert2);
               nubtickoff[vert2]=1;
          }
       }
    }

    protected void writeNub(Shape s,int pointindex){
        double a = s.points[pointindex][0];
        double b = s.points[pointindex][1];
        double c = s.points[pointindex][2];

        output.println("Separator { ");

        if (s.colorcoded){
            output.println(colorstrings.getcolor(s, pointindex));
        } else {
            output.println(colorstrings.getcolor(s.vertcolor));
        }
        output.println("  Translation { ");
        output.println("      translation "+a+" "+b+" "+c);
        output.println("  }");

        output.println("  Sphere { ");
        output.println("      radius " + s.vertradius);
        output.println("  }");
        output.println("}");
    }

    protected void writeCylinder(Shape s,int i){

        double transx = vrmlcyls[i][0];
        double transy = vrmlcyls[i][1];
        double transz = vrmlcyls[i][2];
        double length = vrmlcyls[i][3];
        double roty   = vrmlcyls[i][4];
        double rotx   = vrmlcyls[i][5];

        output.println("Separator { ");

        output.println(colorstrings.getcolor(s.edgecolor));

        output.println("  Translation { ");
        output.println("      translation "+transx+" "+transy+" "+transz);
        output.println("  }");
        if (roty!=0){
          output.println("  Rotation { ");
          output.println("      rotation     0.0 1.0 0.0 "+roty);
          output.println("  }");
        }
        if (rotx!=0){
          output.println("  Rotation { ");
          output.println("      rotation     1.0 0.0 0.0 "+rotx);
          output.println("  }");
        }

        output.println("   Cylinder{ ");
        output.println("      radius " + s.cylradius);
        output.println("      height " + length);
        output.println("   }");

        output.println("}");
    }

    protected void writeMesh(Shape s){
        output.println("Coordinate3 {");
        output.println("      point [");

        for(int i = 0;i<s.points.length;i++){
            output.println(s.points[i][0]+" "+s.points[i][1]+" "+s.points[i][2]+",");}

        output.println("              ]");
        output.println("}");
        output.println("");
        output.println("IndexedLineSet {");
        output.println("   coordIndex [");
        
        for(int i = 0;i<s.edges.length;i++){
            output.println("          " +
                             s.edges[i][0] + ", "+
                             s.edges[i][1] + ", -1,");}
            
        output.println("              ]");
        output.println("}");            
    }

    protected void writesphere(Shape s){

       output.println("Separator { ");

       output.println(colorstrings.getcolor(s.edgecolor));

       output.println("   Translation {");
       output.println("      translation "+ s.pivot[0]+" "+
                                            s.pivot[1]+" "+
                                            s.pivot[2]);
       output.println("   }");
       output.println("   Sphere {");
       output.println("       radius "+s.radius);
       output.println("   }");

       output.println("}");
    }

    public static void main(String[] args){}
}


class Vrmlops {
    private static final double DEG2RAD = Math.PI/180;
    private static final double RAD2DEG = 180/Math.PI;

    public static double[][] vrmlcalcs(Shape s){
          double hyp = 0;
          double xzhyp = 0;
          double[] tip = new double[3];
          double[][] vrmlcyls = new double[s.noedges][6];

          for (int i=0;i<s.edges.length;i++){
             int vert1 = s.edges[i][0];
             int vert2 = s.edges[i][1];

             for (int j=0;j<3;j++){
                // midpoint of edge between vert1 and vert2
                vrmlcyls[i][j] = (s.points[vert1][j] + s.points[vert2][j])/2.0;

                // tip = translated endpoint (w/ midpoint at 0,0,0)
                tip[j] = s.points[vert2][j] - vrmlcyls[i][j];
             }

             // distance from translated midpoint to
             // translated endpoint
             hyp =
             Math.sqrt(tip[0]*tip[0] + tip[1]*tip[1] + tip[2]*tip[2]);
             vrmlcyls[i][3] = 2.0*hyp; // full length of cylinder
             xzhyp = Math.sqrt(tip[0]*tip[0] + tip[2]*tip[2]);

             // rotate zx plane around north/south (y) axis to
             // have xy plane intersect tip
             if (xzhyp!=0){
                vrmlcyls[i][4] = Math.acos(tip[2]/xzhyp);
                if (tip[0]<=0) vrmlcyls[i][4] = - vrmlcyls[i][4];
             }

             // rotate about x axis, tilting y axis to tip
             if (hyp!=0){ vrmlcyls[i][5] = Math.acos(tip[1]/hyp);}
          }

          return vrmlcyls;   //object handle for array w/ new data
    }
}

class ColorDict {

    public static Hashtable colors = new Hashtable();

    ColorDict() {
       colors.put("Red",    "1.0 0.0 0.0");
       colors.put("Green",  "0.0 1.0 0.0");
       colors.put("Blue",   "0.0 0.0 1.0");
       colors.put("Yellow", "1.0 1.0 0.0");
       colors.put("Cyan",   "0.0 1.0 1.0");
       colors.put("Magenta","1.0 0.0 1.0");
       colors.put("White",  "1.0 1.0 1.0");
       colors.put("Black",  "0.0 0.0 0.0");
       colors.put("Orange", "1.0 0.5 0.0");
       colors.put("Violet", "0.309804 0.184314 0.309804");
       colors.put("Indigo", "0.294117 0.0      0.509803");
       colors.put("Brown",  "0.647059 0.164706 0.164706");
       colors.put("Grey",   "0.752941 0.752941 0.752941");
       colors.put("Gray",   "0.752941 0.752941 0.752941");
    }

    public String getcolor(String cname){
       String rgb;
       if (colors.containsKey(cname)) {
           rgb = (String)colors.get(cname);
           rgb = "   Material { diffuseColor " + rgb + "}";
         }
       else
         {rgb = "   Material { diffuseColor 0.0 0.0 0.0 " + "}";}

       return rgb;}

    public String getcolor(Shape s, int rgbindex){
       String rgb;
       rgb = ""+s.rgbcodes[rgbindex][0]+" "+s.rgbcodes[rgbindex][1]+" "+s.rgbcodes[rgbindex][2];
       rgb = "   Material { diffuseColor " + rgb + "}";
       return rgb;}
}

