//:Writers.java -- defines VRML and POV subclasses of DataWriter
package com.pdx4d.polys;

import java.io.*;

abstract class Datawriter {
    protected static String outfile = new String("outfile.txt");
    protected static boolean fileopen = false;
    public static double cyldiam = 0.04;
    protected static PrintWriter output;

    static protected PrintWriter openfile(String filename){
    try{
        System.out.println("Creating " + filename);
        output =
        new PrintWriter(
           new BufferedWriter(
             new FileWriter(filename)));
      } catch(Exception e)
         {System.out.println("Couldn't create file!");}
      return output;
    }
    
    static public void writedone(){
        System.out.println("Close data file");
        output.close();
    }    
}

class POVwriter extends Datawriter {
    protected static String outfile = new String("outfile.pov");
    public static double camfactor = 8;
    protected static PrintWriter output;

    static public void writeshape(Shape s){
        String shapetype = (s.getClass()).getName();
        System.out.println
        ("Writing object of type: " +shapetype + " to .pov file");
        if (!fileopen)
           {output = openfile(outfile);
           fileopen = true;
           writeheader();}
        if (shapetype.equals("com.pdx4d.polys.Sphere")){
           writesphere(s);}
        else {
           writeEdges(s);}
    }

    static protected void writeheader(){
      System.out.println("Write POV header");
      output.println("//POV-Ray script");
      output.println("#version 3.1;");
      output.println("global_settings { assumed_gamma 2.2 }");
      output.println("#include \"colors.inc\"");
      output.println("");
      output.println("#declare Cam_factor = " + camfactor + ";");
      output.println("#declare Camera_X = 1 * Cam_factor;");
      output.println("#declare Camera_Y = 0.5 * Cam_factor;");
      output.println("#declare Camera_Z = -0.9 * Cam_factor;");
      output.println("camera { location  <Camera_X, Camera_Y, Camera_Z>");
      output.println("            up        <0, 1.0,  0>    right     <-4/3, 0,  0>");
      output.println("            direction <0, 0,  3>      look_at   <0, 0, 0> ");
      output.println("            rotate <0,0,0>}");
      output.println("");
      output.println("light_source { <Camera_X - 2, Camera_Y + 5 , Camera_Z + 5> color White }");
      output.println("light_source { <Camera_X - 2, Camera_Y + 5 , Camera_Z - 3> color White }");
      output.println("");
      output.println("// Background:");
      output.println("background {color White}");    
    }

    static protected void writeEdges(Shape s){
    for (int i=0;i<s.edges.length;i++){
       writeNub(s,s.edges[i][0]);
       writeCylinder(s,s.edges[i][0],s.edges[i][1]);
       writeNub(s,s.edges[i][1]);
       }
    }

    static protected void writeNub(Shape s,int pointindex){
        double a = s.points[pointindex][0];
        double b = s.points[pointindex][1];
        double c = s.points[pointindex][2];
        output.println("sphere{<" + a + "," + b + "," + c +">," + cyldiam
                + " pigment {color "+ s.shapecolor + "} no_shadow}");
    }

    static protected void writeCylinder
        (Shape s,int startpoint, int endpoint){

        double a = s.points[startpoint][0];
        double b = s.points[startpoint][1];
        double c = s.points[startpoint][2];
        double d = s.points[endpoint][0];
        double e = s.points[endpoint][1];
        double f = s.points[endpoint][2];

        output.println(
            "cylinder{<"+ a +"," + b + "," + c +">,<"
                        + d +"," + e + "," + f +">," + cyldiam
                        + " pigment {color " + s.shapecolor + "} no_shadow}");        
    }

    static protected void writesphere(Shape s){
    output.println("sphere { <" + s.pivot[0]+", "
                                + s.pivot[1]+", "
                                + s.pivot[2]+">, " + s.radius);                                
    output.println("      texture {");
    output.println("        pigment {");
    output.println("           wood");
    output.println("           color_map {");
    output.println("            [0.0 color DarkTan]");
    output.println("            [0.9 color DarkBrown]");
    output.println("            [1.0 color VeryDarkBrown]");
    output.println("                     }");
    output.println("        turbulence 0.08");
    output.println("                 }");
    output.println("      finish { phong .5 }");
    output.println("              }");
    output.println("       }");
    }

}

class VRMLwriter extends Datawriter {
    public static String outfile = new String("outfile.wrl");
    public static PrintWriter output;

    static public void writeshape(Shape s){
       String shapetype = (s.getClass()).getName();
       System.out.println
       ("Writing object of type: " +shapetype + " to .wrl file");
       if (!fileopen)
           {output = openfile(outfile);
           fileopen = true;
           writeheader();}
       if (shapetype.equals("com.pdx4d.polys.Sphere")){
           writesphere(s);}
       else {
           writeEdges(s);}
    }

    static protected void writeheader(){
      System.out.println("Write VRML header");
      output.println("#VRML V1.0 ascii");
      output.println("DEF BackgroundColor Info { string \"0.0 0.0 1.0\" }");
      output.println("Material { diffuseColor 1 0 0}  #end Material set to blue");
    }

    static protected void writeEdges(Shape s){
        output.println("Coordinate3 {");
        output.println("      point [");

        for(int i = 0;i<s.points.length;i++){
            output.println(s.points[i][0]+" "+s.points[i][1]+" "+s.points[i][2]+",");}

        output.println("              ]");
        output.println("}");
        output.println("");
        output.println("IndexedLineSet {");
        output.println("   coordIndex [");
        
        for(int i = 0;i<s.edges.length;i++){
            output.println("          " +
                             s.edges[i][0] + ", "+
                             s.edges[i][1] + ", -1,");}
            
        output.println("              ]");
        output.println("}");            
    }

    static protected void writesphere(Shape s){
       output.println("Translation {");
       output.println("   translation "+ s.pivot[0]+" "+
                                         s.pivot[1]+" "+
                                         s.pivot[2]);
       output.println("}");
       output.println("Sphere {");
       output.println("   radius "+s.radius);
       output.println("}");

       output.println("Translation {");
       output.println("   translation "+ -s.pivot[0]+" "+
                                         -s.pivot[1]+" "+
                                         -s.pivot[2]);
       output.println("}");
    }
}

public class Writers {
    public static void main(String[] args){}
}
